@@ -7,13 +7,13 @@
 //===----------------------------------------------------------------------===//
 
 #include "src/math/lroundl.h"
+#include "src/__support/FPUtil/NearestIntegerOperations.h"
 #include "src/__support/common.h"
-#include "utils/FPUtil/NearestIntegerOperations.h"
 
 namespace __llvm_libc {
 
 LLVM_LIBC_FUNCTION(long, lroundl, (long double x)) {
-  return fputil::roundToSignedInteger<long double, long>(x);
+  return fputil::round_to_signed_integer<long double, long>(x);
 }
 
 } // namespace __llvm_libc
