@@ -13,8 +13,6 @@ from lldbsuite.test import lldbutil
 
 class ProcessListTestCase(TestBase):
 
-    mydir = TestBase.compute_mydir(__file__)
-
     NO_DEBUG_INFO_TESTCASE = True
 
     @skipIfWindows  # https://bugs.llvm.org/show_bug.cgi?id=43702
@@ -29,9 +27,4 @@ class ProcessListTestCase(TestBase):
 
         substrs = [str(popen.pid), "TestProcess arg1 --arg2 arg3"]
 
-        # Because LLDB isn't the one spawning the subprocess, the PID will be
-        # different during replay.
-        if configuration.is_reproducer_replay():
-            substrs.pop(0)
-
         self.expect("platform process list -v", substrs=substrs)
